@@ -36,7 +36,7 @@ before "proxy-server" and add the following filter in the file:
     # set topic
     topic = notifications
     # skip metering of requests from listed project ids
-    ignore_projects = <proj_uuid>, <proj_uuid2>
+    ignore_projects = <proj_uuid>, <proj_uuid2>, <proj_name>
     # Whether to send events to messaging driver in a background thread
     nonblocking_notify = False
     # Queue size for sending notifications in background thread (0=unlimited).
@@ -44,13 +44,30 @@ before "proxy-server" and add the following filter in the file:
     send_queue_size = 1000
     # Logging level control
     log_level = WARNING
+
+    # All keystoneauth1 options can be set to query project name for
+    # ignore_projects option, here is just a example:
+    auth_type = password
+    auth_url = https://[::1]:5000
+    project_name = service
+    project_domain_name = Default
+    username = user
+    user_domain_name = Default
+    password = a_big_secret
+    interface = public
 """
+import datetime
 import functools
 import logging
 
+from keystoneauth1 import exceptions as ksa_exc
+from keystoneauth1.loading import adapter as ksa_adapter
+from keystoneauth1.loading import base as ksa_base
+from keystoneauth1.loading import session as ksa_session
+from keystoneclient.v3 import client as ks_client
 from oslo_config import cfg
 import oslo_messaging
-from oslo_utils import timeutils
+from oslo_utils import strutils
 from pycadf import event as cadf_event
 from pycadf.helper import api
 from pycadf import measurement as cadf_measurement
@@ -61,7 +78,17 @@ import six.moves.queue as queue
 import six.moves.urllib.parse as urlparse
 import threading
 
-_LOG = logging.getLogger(__name__)
+
+LOG = logging.getLogger(__name__)
+
+
+def list_from_csv(comma_separated_str):
+    if comma_separated_str:
+        return list(
+            filter(lambda x: x,
+                   map(lambda x: x.strip(),
+                       comma_separated_str.split(','))))
+    return []
 
 
 def _log_and_ignore_error(fn):
@@ -70,8 +97,8 @@ def _log_and_ignore_error(fn):
         try:
             return fn(*args, **kwargs)
         except Exception as e:
-            _LOG.exception('An exception occurred processing '
-                           'the API call: %s ', e)
+            LOG.exception('An exception occurred processing '
+                          'the API call: %s ', e)
     return wrapper
 
 
@@ -104,6 +131,23 @@ class InputProxy(object):
         self.bytes_received += len(line)
         return line
 
+    def close(self):
+        close_method = getattr(self.wsgi_input, 'close', None)
+        if callable(close_method):
+            close_method()
+
+
+class KeystoneClientLoader(ksa_adapter.Adapter):
+    """Keystone client adapter loader.
+
+    Keystone client and Keystoneauth1 adapter take exactly the same options, so
+    it's safe to create a keystone client with keystoneauth adapter options.
+    """
+
+    @property
+    def plugin_class(self):
+        return ks_client.Client
+
 
 class Swift(object):
     """Swift middleware used for counting requests."""
@@ -111,19 +155,21 @@ class Swift(object):
     event_queue = None
     threadLock = threading.Lock()
 
+    DEFAULT_IGNORE_PROJECT_NAMES = ['service']
+
     def __init__(self, app, conf):
         self._app = app
-        self.ignore_projects = [
-            proj.strip() for proj in
-            conf.get('ignore_projects', 'gnocchi').split(',')]
+
+        self.ignore_projects = self._get_ignore_projects(conf)
 
         oslo_messaging.set_transport_defaults(conf.get('control_exchange',
                                                        'swift'))
         self._notifier = oslo_messaging.Notifier(
-            oslo_messaging.get_transport(cfg.CONF, url=conf.get('url')),
+            oslo_messaging.get_notification_transport(cfg.CONF,
+                                                      url=conf.get('url')),
             publisher_id='ceilometermiddleware',
             driver=conf.get('driver', 'messagingv2'),
-            topic=conf.get('topic', 'notifications'))
+            topics=[conf.get('topic', 'notifications')])
 
         self.metadata_headers = [h.strip().replace('-', '_').lower()
                                  for h in conf.get(
@@ -134,7 +180,7 @@ class Swift(object):
         if self.reseller_prefix and self.reseller_prefix[-1] != '_':
             self.reseller_prefix += '_'
 
-        _LOG.setLevel(getattr(logging, conf.get('log_level', 'WARNING')))
+        LOG.setLevel(getattr(logging, conf.get('log_level', 'WARNING')))
 
         # NOTE: If the background thread's send queue fills up, the event will
         #  be discarded
@@ -142,7 +188,8 @@ class Swift(object):
         # For backward compatibility we default to False and therefore wait for
         #  sending to complete. This causes swift proxy to hang if the
         #  destination is unavailable.
-        self.nonblocking_notify = conf.get('nonblocking_notify', False)
+        self.nonblocking_notify = strutils.bool_from_string(
+            conf.get('nonblocking_notify', False))
 
         # Initialize the sending queue and thread, but only once
         if self.nonblocking_notify and Swift.event_queue is None:
@@ -150,11 +197,57 @@ class Swift(object):
             if Swift.event_queue is None:
                 send_queue_size = int(conf.get('send_queue_size', 1000))
                 Swift.event_queue = queue.Queue(send_queue_size)
-                Swift.event_sender = SendEventThread(self._notifier)
-                Swift.event_sender.start()
-                _LOG.debug('Started sender thread')
+                self.start_sender_thread()
             Swift.threadLock.release()
 
+    def _get_ignore_projects(self, conf):
+        if 'auth_type' not in conf:
+            LOG.info("'auth_type' is not set assuming ignore_projects are "
+                     "only project uuid.")
+            return list_from_csv(conf.get('ignore_projects'))
+
+        if 'ignore_projects' in conf:
+            ignore_projects = list_from_csv(conf.get('ignore_projects'))
+        else:
+            ignore_projects = self.DEFAULT_IGNORE_PROJECT_NAMES
+
+        if not ignore_projects:
+            return []
+
+        def opt_getter(opt):
+            # TODO(sileht): This method does not support deprecated opt names
+            val = conf.get(opt.name)
+            if val is None:
+                val = conf.get(opt.dest)
+            return val
+
+        auth_type = conf.get('auth_type')
+        plugin = ksa_base.get_plugin_loader(auth_type)
+
+        auth = plugin.load_from_options_getter(opt_getter)
+        session = ksa_session.Session().load_from_options_getter(
+            opt_getter, auth=auth)
+        client = KeystoneClientLoader().load_from_options_getter(
+            opt_getter, session=session)
+
+        projects = []
+        for name_or_id in ignore_projects:
+            projects.extend(self._get_keystone_projects(client, name_or_id))
+        return projects
+
+    @staticmethod
+    def _get_keystone_projects(client, name_or_id):
+        try:
+            return [client.projects.get(name_or_id)]
+        except ksa_exc.NotFound:
+            pass
+        if isinstance(name_or_id, six.binary_type):
+            name_or_id = name_or_id.decode('utf-8', 'strict')
+        projects = client.projects.list(name=name_or_id)
+        if not projects:
+            LOG.warning("fail to find project '%s' in keystone", name_or_id)
+        return [p.id for p in projects]
+
     def __call__(self, env, start_response):
         start_response_args = [None]
         input_proxy = InputProxy(env['wsgi.input'])
@@ -179,8 +272,14 @@ class Swift(object):
                 while chunk:
                     bytes_sent += len(chunk)
                     yield chunk
-                    chunk = next(iterator)
+                    try:
+                        chunk = next(iterator)
+                    except StopIteration:
+                        chunk = ''
             finally:
+                close_method = getattr(iterable, 'close', None)
+                if callable(close_method):
+                    close_method()
                 self.emit_event(env, input_proxy.bytes_received, bytes_sent)
 
         try:
@@ -193,13 +292,14 @@ class Swift(object):
 
     @_log_and_ignore_error
     def emit_event(self, env, bytes_received, bytes_sent, outcome='success'):
-        if ((env.get('HTTP_X_SERVICE_PROJECT_ID') or
-                env.get('HTTP_X_PROJECT_ID') or
-                env.get('HTTP_X_TENANT_ID')) in self.ignore_projects or
-                env.get('swift.source') is not None):
+        if (
+                (env.get('HTTP_X_SERVICE_PROJECT_ID')
+                 or env.get('HTTP_X_PROJECT_ID')
+                 or env.get('HTTP_X_TENANT_ID')) in self.ignore_projects
+                or env.get('swift.source') is not None):
             return
 
-        path = urlparse.quote(env['PATH_INFO'])
+        path = urlparse.quote(env.get('swift.backend_path', env['PATH_INFO']))
         method = env['REQUEST_METHOD']
         headers = {}
         for header in env:
@@ -231,7 +331,7 @@ class Swift(object):
         except ValueError:
             return
 
-        now = timeutils.utcnow().isoformat()
+        now = datetime.datetime.utcnow().isoformat()
 
         resource_metadata = {
             "path": path,
@@ -256,8 +356,8 @@ class Swift(object):
         initiator = cadf_resource.Resource(
             typeURI='service/security/account/user',
             id=env.get('HTTP_X_USER_ID'))
-        initiator.project_id = (env.get('HTTP_X_PROJECT_ID') or
-                                env.get('HTTP_X_TENANT_ID'))
+        initiator.project_id = (env.get('HTTP_X_PROJECT_ID')
+                                or env.get('HTTP_X_TENANT_ID'))
 
         # build notification body
         event = cadf_event.Event(eventTime=now, outcome=outcome,
@@ -280,12 +380,21 @@ class Swift(object):
         if self.nonblocking_notify:
             try:
                 Swift.event_queue.put(event, False)
-                _LOG.debug('Event %s added to send queue', event.id)
+                if not Swift.event_sender.is_alive():
+                    Swift.threadLock.acquire()
+                    self.start_sender_thread()
+                    Swift.threadLock.release()
+
             except queue.Full:
-                _LOG.warning('Send queue FULL: Event %s not added', event.id)
+                LOG.warning('Send queue FULL: Event %s not added', event.id)
         else:
             Swift.send_notification(self._notifier, event)
 
+    def start_sender_thread(self):
+        Swift.event_sender = SendEventThread(self._notifier)
+        Swift.event_sender.daemon = True
+        Swift.event_sender.start()
+
     @staticmethod
     def send_notification(notifier, event):
         notifier.info({}, 'objectstore.http.request', event.as_dict())
@@ -296,19 +405,18 @@ class SendEventThread(threading.Thread):
     def __init__(self, notifier):
         super(SendEventThread, self).__init__()
         self.notifier = notifier
-        self.daemon = True
 
     def run(self):
         """Send events without blocking swift proxy."""
         while True:
             try:
-                _LOG.debug('Wait for event from send queue')
+                LOG.debug('Wait for event from send queue')
                 event = Swift.event_queue.get()
-                _LOG.debug('Got event %s from queue - now send it', event.id)
+                LOG.debug('Got event %s from queue - now send it', event.id)
                 Swift.send_notification(self.notifier, event)
-                _LOG.debug('Event %s sent.', event.id)
-            except Exception:
-                _LOG.exception("SendEventThread loop exception")
+                LOG.debug('Event %s sent.', event.id)
+            except BaseException:
+                LOG.exception("SendEventThread loop exception")
 
 
 def filter_factory(global_conf, **local_conf):
